"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.startFoundationSimulationServer = exports.createFoundationSimulationServer = void 0;
const express_1 = __importDefault(require("express"));
const lodash_1 = require("lodash");
const openapi_backend_1 = __importDefault(require("openapi-backend"));
const ajv_formats_1 = __importDefault(require("ajv-formats"));
const index_1 = require("./store/index");
function createFoundationSimulationServer({ port = 9000, openapi, extendStore, extendRouter, }) {
    return () => {
        let app = (0, express_1.default)();
        app.use(express_1.default.json());
        let simulationStore = (0, index_1.createSimulationStore)(extendStore);
        if (extendRouter) {
            extendRouter(app, simulationStore);
        }
        if (openapi) {
            for (let spec of openapi) {
                let { document, handlers, apiRoot } = spec;
                let mergedOAS = Array.isArray(document)
                    ? mergeDocumentArray(document)
                    : document;
                let api = new openapi_backend_1.default({
                    definition: mergedOAS,
                    apiRoot,
                    customizeAjv: (ajv) => {
                        (0, ajv_formats_1.default)(ajv, {
                            mode: "fast",
                            formats: ["email", "uri", "date-time", "uuid"],
                        });
                        return ajv;
                    },
                });
                // register your framework specific request handlers here
                let handlerObjectRegistration = (handlerEntries, prefix) => {
                    for (let [key, handler] of Object.entries(handlerEntries)) {
                        if (typeof handler === "object") {
                            handlerObjectRegistration(handler, key);
                        }
                        else {
                            api.register(`${prefix ? `${prefix}/` : ""}${key}`, handler);
                        }
                    }
                };
                handlerObjectRegistration(handlers(simulationStore));
                api.register({
                    validationFail: (c, req, res) => res.status(400).json({ err: c.validation.errors }),
                    // if route not in API, continue to next API or `app.all('*')` fallback
                    notFound: (c, req, res, next) => next(),
                    notImplemented: (c, req, res) => {
                        let { status, mock } = c.api.mockResponseForOperation(
                        // the route validates this exists and throws if it does not
                        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                        c.operation.operationId);
                        return res.status(status).json(mock);
                    },
                });
                // initalize the backend
                api.init();
                app.use((req, res, next) => api.handleRequest(req, req, res, next));
            }
        }
        // if no extendRouter routes or openapi routes handle this, return 404
        app.all("*", (req, res) => res.status(404).json({ error: "not found" }));
        return {
            listen: async (portOverride, callback) => {
                let listeningPort = portOverride !== null && portOverride !== void 0 ? portOverride : port;
                let server = app.listen(listeningPort, callback);
                if (!server.listening) {
                    await new Promise((resolve) => {
                        server.once("listening", resolve);
                    });
                }
                return {
                    server,
                    ensureClose: async () => {
                        await new Promise((resolve) => {
                            server.once("close", resolve);
                            server.closeIdleConnections();
                            server.closeAllConnections();
                            server.close();
                        });
                        // it takes a bit to close the server, but there is not method
                        //  that will cleanly await that process
                        await new Promise((resolve) => setTimeout(resolve, 20));
                    },
                };
            },
        };
    };
}
exports.createFoundationSimulationServer = createFoundationSimulationServer;
const mergeDocumentArray = (documents) => {
    let document = (0, lodash_1.merge)({}, ...documents);
    return document;
};
async function startFoundationSimulationServer(arg) {
    let simulation = createFoundationSimulationServer(arg)();
    return simulation.listen();
}
exports.startFoundationSimulationServer = startFoundationSimulationServer;
//# sourceMappingURL=index.js.map